/*
 * Humidor fan controller.
 * Written by: Richard D Wickstrom
 * Version: 1.0
 * Date: 05 Sep 2017
 * 
 * This program controls a pair of fans that spread humidity in a cigar humidor. It implements a simple bang-bang
 * controller that reads the humidity from a pair of DHT22 humidity/temperature sensors. It displays the data on
 * this OLED display: "Diymall 0.96" Inch I2c IIC Serial 128x64 Oled LCD LED White Display Module"
 * This uses an Arduino Uno. I tried a Leonardo, but the display didn't work - I didn't try to figure out why.
 */
 

#include <U8g2lib.h>
#include <U8x8lib.h>

// DHT depends on the following Arduino libraries:
// - Adafruit Unified Sensor Library: https://github.com/adafruit/Adafruit_Sensor
// - DHT Sensor Library: https://github.com/adafruit/DHT-sensor-library

#include <Adafruit_Sensor.h>
#include <DHT.h>
#include <DHT_U.h>
#include <Bounce.h>
#include <stdio.h>
#include <SimpleTimer.h>
#include <SoftwareSerial.h>

//Prototypes
void dumpSensorTemperatureInfo( sensor_t *);
void dumpSensorHumidityInfo( sensor_t *);

#define DHT1PIN            2         // Pin which is connected to the DHT sensor.
#define DHT2PIN            3         // Pin which is connected to the DHT sensor.
#define DEBUG_RX_PIN 5 //"Rx" is recieve _to_ the Arduno
#define DEBUG_TX_PIN 4 //"Tx" is transmit _from_ the Arduno

#define DHTTYPE           DHT22     // DHT 22 (AM2302)
#define BUFF_SIZE 18
#define UP_BUTTON_PIN 8
#define UP_BUTTON_LED_PIN 9
#define DOWN_BUTTON_PIN 10
#define DOWN_BUTTON_LED_PIN 11
#define FAN1_PIN 12
#define FAN2_PIN 13

// See guide for details on sensor wiring and usage:
//   https://learn.adafruit.com/dht/overview

DHT_Unified dht1(DHT1PIN, DHTTYPE);
DHT_Unified dht2(DHT2PIN, DHTTYPE);

//Constructor for display
U8G2_SSD1306_128X64_NONAME_1_HW_I2C u8g2(U8G2_R0);

//Creat the timer object
SimpleTimer timer;
SoftwareSerial debugSS(DEBUG_RX_PIN, DEBUG_TX_PIN);

char buff[BUFF_SIZE];

unsigned int setPoint = 70; //Starting setpoint for humidity is 70%

bool fan1State = false;
bool fan2State = true;

Bounce upButton = Bounce(UP_BUTTON_PIN, 10);  // 10 ms debounce
Bounce downButton = Bounce(DOWN_BUTTON_PIN, 10);  // 10 ms debounce

float humidity1=100.0;
float humidity2=100.0;
float temperature1=0.0;
float temperature2=0.0;

void setup() {
  debugSS.begin(9600);
  
  // Initialize device.
  dht1.begin();
  dht2.begin();

  //Sample sensors every 2 seconds
  timer.setInterval(2000, readSensors);

  pinMode(UP_BUTTON_PIN, INPUT_PULLUP);
  pinMode(UP_BUTTON_LED_PIN, OUTPUT);
  digitalWrite(UP_BUTTON_LED_PIN, true);

  pinMode(DOWN_BUTTON_PIN, INPUT_PULLUP); 
  pinMode(DOWN_BUTTON_LED_PIN, OUTPUT); 
  digitalWrite(DOWN_BUTTON_LED_PIN, true);

  //Fan control pins
  pinMode(FAN1_PIN, OUTPUT); 
  digitalWrite(FAN1_PIN, false);
  pinMode(FAN2_PIN, OUTPUT); 
  digitalWrite(FAN2_PIN, false);
  
  // Diagnostic: Print temperature and humidity sensor details.
  sensor_t sensor;

  dht1.temperature().getSensor(&sensor);
  dumpSensorTemperatureInfo(&sensor);
  dht1.humidity().getSensor(&sensor);
  dumpSensorHumidity(&sensor);

  dht2.temperature().getSensor(&sensor);
  dumpSensorTemperatureInfo(&sensor);
  dht2.humidity().getSensor(&sensor);
  dumpSensorHumidity(&sensor);

  //Start the display class
  u8g2.begin();
}//End of setup

void loop() {
  unsigned long currentMillis = millis();
  upButton.update(); //Needed for the Bounce class  
  downButton.update(); //Needed for the Bounce class  
  timer.run();  

  //Check the buttons
  if(upButton.fallingEdge()){  
    setPoint++; setPoint = setPoint%100;
    sprintf(buff, "Setpoint: %d%%", setPoint);
  }
  if(downButton.fallingEdge()){  
    setPoint--; setPoint = setPoint%100;
    sprintf(buff, "Setpoint: %d%%", setPoint);
  }

  //Turn the fans on if humidity too low
  fan1State = (humidity1<setPoint) ? true : false;
  fan2State = (humidity2<setPoint) ? true : false;
  digitalWrite(FAN1_PIN, fan1State);
  digitalWrite(UP_BUTTON_LED_PIN, fan1State);
  digitalWrite(FAN2_PIN, fan2State);
  digitalWrite(DOWN_BUTTON_LED_PIN, fan2State);

  //Fonts for display
  uint8_t *headerFont =  u8g2_font_ncenB08_tr;
  uint8_t *humidityFont =  u8g2_font_ncenR14_tr  ;
  uint8_t *temperatureFont =  u8g2_font_ncenR10_tf;
  uint8_t *setpointFont =  u8g2_font_5x7_tr;
    
  int humidityYoffset=31;  
  int temperatureYoffset=50; 
  int sens1Xoffset=128/4;     //Center justified
  int sens2Xoffset=3*128/4; //Center justified
  int strWidth;    

  u8g2.firstPage();
  do {
    u8g2.setFont(headerFont);
    u8g2.drawStr(0,10,"  Vic's Daily Humidor");
    
    //Display the measurements
    if(temperature1==0.0)
      {
      u8g2.setFont(humidityFont);
      u8g2.drawStr(sens1Xoffset,humidityYoffset,"Err");
      }
    else
      {
      sprintf(buff, "%d.%d%%", 
          (int)humidity1, (int)(humidity1*10)%10);
      u8g2.setFont(humidityFont);
      strWidth = u8g2.getStrWidth(buff);
      u8g2.drawStr(sens1Xoffset-strWidth/2,humidityYoffset,buff);        

      sprintf(buff, "%d.%d\xB0", 
          (int)temperature1, (int)(temperature1*10)%10);
      u8g2.setFont(temperatureFont);
      strWidth = u8g2.getStrWidth(buff);
      u8g2.drawStr(sens1Xoffset-strWidth/2,temperatureYoffset,buff);        
      }

    if(temperature2==0.0)
      {
      u8g2.setFont(humidityFont);
      u8g2.drawStr(sens2Xoffset,humidityYoffset,"Err");
      }
    else
      {
      sprintf(buff, "%d.%d%%", 
          (int)humidity2, (int)(humidity2*10)%10);
      u8g2.setFont(humidityFont);
      strWidth = u8g2.getStrWidth(buff);
      u8g2.drawStr(sens2Xoffset-strWidth/2,humidityYoffset,buff);        

      sprintf(buff, "%d.%d\xB0", 
          (int)temperature2, (int)(temperature2*10)%10);
      u8g2.setFont(temperatureFont);
      strWidth = u8g2.getStrWidth(buff);
      u8g2.drawStr(sens2Xoffset-strWidth/2,temperatureYoffset,buff);        
      }

    int top = 16;
    u8g2.drawVLine(64, top, 64-top-11);

    u8g2.setFont(setpointFont);
    //Display the setpoint
    sprintf(buff, "Setpoint: %d%%", setPoint);
      strWidth = u8g2.getStrWidth(buff);
    u8g2.drawStr(64-strWidth/2,63,buff);

  } while ( u8g2.nextPage() );

}//End of loop

/////////////////////
/////////////////////

/*
 * Get the measurements once per 2 seconds. This is a callback attached in the setup section
 */
void readSensors() {
  readSensorTempAndHumidity(dht1, &temperature1, &humidity1);
  readSensorTempAndHumidity(dht2, &temperature2, &humidity2);
}

/*
 * Read the temperature and humidity for the sensor. Returns zero if no error.
 */
int readSensorTempAndHumidity(DHT_Unified dht, float *temperature, float *humidity)
{
  sensors_event_t event; 
     
  debugSS.println("---");

  //Assume failure to initialize
  *temperature = 0.0;
  *humidity = 100.0; //Return high value so fan does NOT turn on 

  //Temperature
  dht.temperature().getEvent(&event);
  if (isnan(event.temperature)) {
    debugSS.println("Error reading temperature!");
    return(1);
  }
  else {
    debugSS.print("Temperature: ");
    debugSS.print(event.temperature);
    debugSS.println(" *C");
    *temperature = event.temperature*1.8+32.0;
  }
  
  //Humidity
  dht.humidity().getEvent(&event);
  if (isnan(event.relative_humidity)) {
    debugSS.println("Error reading humidity!");
    *temperature = 0.0; //Zero out temp, since this is used above as error condition    
    return(1);
  }
  else {
    debugSS.print("Humidity: ");
    debugSS.print(event.relative_humidity);
    debugSS.println("%");    
    *humidity = event.relative_humidity;
  }

  return(0);
}

/*
 * Dump temperature sensor info
 */
void dumpSensorTemperatureInfo(sensor_t *sensor)
{
  debugSS.println("------------------------------------");
  debugSS.println("Temperature)");
  debugSS.print  ("Sensor:       "); debugSS.println(sensor->name);
  debugSS.print  ("Driver Ver:   "); debugSS.println(sensor->version);
  debugSS.print  ("Unique ID:    "); debugSS.println(sensor->sensor_id);
  debugSS.print  ("Max Value:    "); debugSS.print(sensor->max_value); debugSS.println(" *C");
  debugSS.print  ("Min Value:    "); debugSS.print(sensor->min_value); debugSS.println(" *C");
  debugSS.print  ("Resolution:   "); debugSS.print(sensor->resolution); debugSS.println(" *C");  
  debugSS.println("------------------------------------");
}
/*
 * Dump humidity sensor info
 */
void dumpSensorHumidity(sensor_t *sensor)
{
  debugSS.println("------------------------------------");
  debugSS.println("Humidity");
  debugSS.print  ("Sensor:       "); debugSS.println(sensor->name);
  debugSS.print  ("Driver Ver:   "); debugSS.println(sensor->version);
  debugSS.print  ("Unique ID:    "); debugSS.println(sensor->sensor_id);
  debugSS.print  ("Max Value:    "); debugSS.print(sensor->max_value); debugSS.println("%");
  debugSS.print  ("Min Value:    "); debugSS.print(sensor->min_value); debugSS.println("%");
  debugSS.print  ("Resolution:   "); debugSS.print(sensor->resolution); debugSS.println("%");  
  debugSS.println("------------------------------------");
}


